/*
 irc-channels-setup.c : irssi

    Copyright (C) 1999-2000 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "module.h"
#include <irssi/src/core/signals.h>
#include <irssi/src/core/channels.h>

void irc_channels_setup_init(void)
{
	signal_add("channel wholist", (SIGNAL_FUNC) channel_send_botcommands);
	signal_add("channel joined", (SIGNAL_FUNC) channel_send_autocommands);
}

void irc_channels_setup_deinit(void)
{
	signal_remove("channel wholist", (SIGNAL_FUNC) channel_send_botcommands);
	signal_remove("channel joined", (SIGNAL_FUNC) channel_send_autocommands);
}
