/*
 * Autogenerated by get-signals.pl from ../../doc/signals.txt,
 * do not edit.
 */

static PERL_SIGNAL_ARGS_REC perl_signal_args[] =
{
	   { "gui exit", { NULL } },
	   { "gui dialog", { "string", "string", NULL } },
	   { "send command", { "string", "iobject", "iobject", NULL } },
	   { "chat protocol created", { "CHAT_PROTOCOL_REC", NULL } },
	   { "chat protocol updated", { "CHAT_PROTOCOL_REC", NULL } },
	   { "chat protocol destroyed", { "CHAT_PROTOCOL_REC", NULL } },
	   { "channel created", { "iobject", "int", NULL } },
	   { "channel destroyed", { "iobject", NULL } },
	   { "chatnet created", { "iobject", NULL } },
	   { "chatnet destroyed", { "iobject", NULL } },
	   { "commandlist new", { "Irssi::Command", NULL } },
	   { "commandlist remove", { "Irssi::Command", NULL } },
	   { "error command", { "int", "string", NULL } },
	   { "send command", { "string", "iobject", "iobject", NULL } },
	   { "send text", { "string", "iobject", "iobject", NULL } },
	   { "command ", { "string", "iobject", "iobject", NULL } },
	   { "default command", { "string", "iobject", "iobject", NULL } },
	   { "ignore created", { "Irssi::Ignore", NULL } },
	   { "ignore destroyed", { "Irssi::Ignore", NULL } },
	   { "ignore changed", { "Irssi::Ignore", NULL } },
	   { "log new", { "Irssi::Log", NULL } },
	   { "log remove", { "Irssi::Log", NULL } },
	   { "log create failed", { "Irssi::Log", NULL } },
	   { "log locked", { "Irssi::Log", NULL } },
	   { "log started", { "Irssi::Log", NULL } },
	   { "log stopped", { "Irssi::Log", NULL } },
	   { "log rotated", { "Irssi::Log", NULL } },
	   { "log written", { "Irssi::Log", "string", NULL } },
	   { "module loaded", { "Irssi::Module", "MODULE_FILE_REC", NULL } },
	   { "module unloaded", { "Irssi::Module", "MODULE_FILE_REC", NULL } },
	   { "module error", { "int", "string", "string", "string", NULL } },
	   { "tls handshake finished", { "iobject", "iobject", NULL } },
	   { "nicklist new", { "iobject", "iobject", NULL } },
	   { "nicklist remove", { "iobject", "iobject", NULL } },
	   { "nicklist changed", { "iobject", "iobject", "string", NULL } },
	   { "nicklist host changed", { "iobject", "iobject", NULL } },
	   { "nicklist account changed", { "iobject", "iobject", "string", NULL } },
	   { "nicklist gone changed", { "iobject", "iobject", NULL } },
	   { "nicklist serverop changed", { "iobject", "iobject", NULL } },
	   { "pidwait", { "int", "int", NULL } },
	   { "query created", { "iobject", "int", NULL } },
	   { "query destroyed", { "iobject", NULL } },
	   { "query nick changed", { "iobject", "string", NULL } },
	   { "window item name changed", { "iobject", NULL } },
	   { "query address changed", { "iobject", NULL } },
	   { "query server changed", { "iobject", "iobject", NULL } },
	   { "rawlog", { "Irssi::Rawlog", "string", NULL } },
	   { "server looking", { "iobject", NULL } },
	   { "server connected", { "iobject", NULL } },
	   { "server connecting", { "iobject", "ulongptr", NULL } },
	   { "server connect failed", { "iobject", NULL } },
	   { "server disconnected", { "iobject", NULL } },
	   { "server quit", { "iobject", "string", NULL } },
	   { "server sendmsg", { "iobject", "string", "string", "int", NULL } },
	   { "setup changed", { NULL } },
	   { "setup reread", { "string", NULL } },
	   { "setup saved", { "string", "int", NULL } },
	   { "ban type changed", { "string", NULL } },
	   { "channel joined", { "iobject", NULL } },
	   { "channel wholist", { "iobject", NULL } },
	   { "channel sync", { "iobject", NULL } },
	   { "channel topic changed", { "iobject", NULL } },
	   { "ctcp msg", { "iobject", "string", "string", "string", "string", NULL } },
	   { "ctcp msg ", { "iobject", "string", "string", "string", "string", NULL } },
	   { "default ctcp msg", { "iobject", "string", "string", "string", "string", NULL } },
	   { "ctcp reply", { "iobject", "string", "string", "string", "string", NULL } },
	   { "ctcp reply ", { "iobject", "string", "string", "string", "string", NULL } },
	   { "default ctcp reply", { "iobject", "string", "string", "string", "string", NULL } },
	   { "ctcp action", { "iobject", "string", "string", "string", "string", NULL } },
	   { "awaylog show", { "Irssi::Log", "int", "int", NULL } },
	   { "server nick changed", { "iobject", NULL } },
	   { "event connected", { "iobject", NULL } },
	   { "server cap ack ", { "iobject", NULL } },
	   { "server cap nak ", { "iobject", NULL } },
	   { "server cap new ", { "iobject", NULL } },
	   { "server cap delete ", { "iobject", NULL } },
	   { "server cap end", { "iobject", NULL } },
	   { "server cap req", { "iobject", "string", NULL } },
	   { "server sasl failure", { "iobject", "string", NULL } },
	   { "server sasl success", { "iobject", NULL } },
	   { "server event", { "iobject", "string", "string", "string", NULL } },
	   { "server event tags", { "iobject", "string", "string", "string", "string", NULL } },
	   { "event ", { "iobject", "string", "string", "string", NULL } },
	   { "default event", { "iobject", "string", "string", "string", NULL } },
	   { "whois default event", { "iobject", "string", "string", "string", NULL } },
	   { "server incoming", { "iobject", "string", NULL } },
	   { "server outgoing modify", { "iobject", "gstring", "int", NULL } },
	   { "redir ", { "iobject", "string", "string", "string", NULL } },
	   { "server lag", { "iobject", NULL } },
	   { "server lag disconnect", { "iobject", NULL } },
	   { "massjoin", { "iobject", "gslist_iobject", NULL } },
	   { "ban new", { "iobject", "Irssi::Irc::Ban", NULL } },
	   { "ban remove", { "iobject", "Irssi::Irc::Ban", "string", NULL } },
	   { "channel mode changed", { "iobject", "string", NULL } },
	   { "nick mode changed", { "iobject", "iobject", "string", "string", "string", NULL } },
	   { "user mode changed", { "iobject", "string", NULL } },
	   { "away mode changed", { "iobject", NULL } },
	   { "netsplit server new", { "iobject", "Irssi::Irc::Netsplitserver", NULL } },
	   { "netsplit server remove", { "iobject", "Irssi::Irc::Netsplitserver", NULL } },
	   { "netsplit new", { "Irssi::Irc::Netsplit", NULL } },
	   { "netsplit remove", { "Irssi::Irc::Netsplit", NULL } },
	   { "dcc ctcp ", { "string", "siobject", NULL } },
	   { "default dcc ctcp", { "string", "siobject", NULL } },
	   { "dcc unknown ctcp", { "string", "string", "string", NULL } },
	   { "dcc reply ", { "string", "siobject", NULL } },
	   { "default dcc reply", { "string", "siobject", NULL } },
	   { "dcc unknown reply", { "string", "string", "string", NULL } },
	   { "dcc chat message", { "siobject", "string", NULL } },
	   { "dcc created", { "siobject", NULL } },
	   { "dcc destroyed", { "siobject", NULL } },
	   { "dcc connected", { "siobject", NULL } },
	   { "dcc rejecting", { "siobject", NULL } },
	   { "dcc closed", { "siobject", NULL } },
	   { "dcc request", { "siobject", "string", NULL } },
	   { "dcc request send", { "siobject", NULL } },
	   { "dcc chat message", { "siobject", "string", NULL } },
	   { "dcc transfer update", { "siobject", NULL } },
	   { "dcc get receive", { "siobject", NULL } },
	   { "dcc error connect", { "siobject", NULL } },
	   { "dcc error file create", { "siobject", "string", NULL } },
	   { "dcc error file open", { "string", "string", "int", NULL } },
	   { "dcc error get not found", { "string", NULL } },
	   { "dcc error send exists", { "string", "string", NULL } },
	   { "dcc error unknown type", { "string", NULL } },
	   { "dcc error close not found", { "string", "string", "string", NULL } },
	   { "autoignore new", { "iobject", "Irssi::Irc::Autoignore", NULL } },
	   { "autoignore remove", { "iobject", "Irssi::Irc::Autoignore", NULL } },
	   { "flood", { "iobject", "string", "string", "int", "string", NULL } },
	   { "notifylist new", { "Irssi::Irc::Notifylist", NULL } },
	   { "notifylist remove", { "Irssi::Irc::Notifylist", NULL } },
	   { "notifylist joined", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "notifylist away changed", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "notifylist left", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "proxy client connecting", { "Irssi::Irc::Client", NULL } },
	   { "proxy client connected", { "Irssi::Irc::Client", NULL } },
	   { "proxy client disconnected", { "Irssi::Irc::Client", NULL } },
	   { "proxy client command", { "Irssi::Irc::Client", "string", "string", NULL } },
	   { "proxy client dump", { "Irssi::Irc::Client", "string", NULL } },
	   { "gui print text", { "Irssi::UI::Window", "int", "int", "int", "string", "Irssi::UI::TextDest", NULL } },
	   { "gui print text finished", { "Irssi::UI::Window", "Irssi::UI::TextDest", NULL } },
	   { "complete word", { "glistptr_string", "Irssi::UI::Window", "string", "string", "intptr", NULL } },
	   { "irssi init read settings", { NULL } },
	   { "exec new", { "Irssi::UI::Process", NULL } },
	   { "exec remove", { "Irssi::UI::Process", "int", NULL } },
	   { "exec input", { "Irssi::UI::Process", "string", NULL } },
	   { "message public", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message private", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message own_public", { "iobject", "string", "string", NULL } },
	   { "message own_private", { "iobject", "string", "string", "string", NULL } },
	   { "message join", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "message part", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message quit", { "iobject", "string", "string", "string", NULL } },
	   { "message kick", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "message nick", { "iobject", "string", "string", "string", NULL } },
	   { "message own_nick", { "iobject", "string", "string", "string", NULL } },
	   { "message invite", { "iobject", "string", "string", "string", NULL } },
	   { "message invite_other", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message topic", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message host_changed", { "iobject", "string", "string", "string", NULL } },
	   { "message account_changed", { "iobject", "string", "string", "string", NULL } },
	   { "message away_notify", { "iobject", "string", "string", "string", NULL } },
	   { "keyinfo created", { "Irssi::UI::Keyinfo", NULL } },
	   { "keyinfo destroyed", { "Irssi::UI::Keyinfo", NULL } },
	   { "print text", { "Irssi::UI::TextDest", "string", "string", NULL } },
	   { "print format", { "Irssi::UI::Theme", "string", "Irssi::UI::TextDest", "formatnum_args", NULL } },
	   { "print noformat", { "Irssi::UI::TextDest", "string", NULL } },
	   { "theme created", { "Irssi::UI::Theme", NULL } },
	   { "theme destroyed", { "Irssi::UI::Theme", NULL } },
	   { "window hilight", { "Irssi::UI::Window", NULL } },
	   { "window hilight check", { "Irssi::UI::TextDest", "string", "intptr", "intptr", NULL } },
	   { "window dehilight", { "Irssi::UI::Window", NULL } },
	   { "window activity", { "Irssi::UI::Window", "int", NULL } },
	   { "window item hilight", { "iobject", NULL } },
	   { "window item activity", { "iobject", "int", NULL } },
	   { "window item new", { "Irssi::UI::Window", "iobject", NULL } },
	   { "window item remove", { "Irssi::UI::Window", "iobject", NULL } },
	   { "window item moved", { "Irssi::UI::Window", "iobject", "Irssi::UI::Window", NULL } },
	   { "window item changed", { "Irssi::UI::Window", "iobject", NULL } },
	   { "window item server changed", { "Irssi::UI::Window", "iobject", NULL } },
	   { "window created", { "Irssi::UI::Window", NULL } },
	   { "window destroyed", { "Irssi::UI::Window", NULL } },
	   { "window changed", { "Irssi::UI::Window", "Irssi::UI::Window", NULL } },
	   { "window changed automatic", { "Irssi::UI::Window", NULL } },
	   { "window server changed", { "Irssi::UI::Window", "iobject", NULL } },
	   { "window refnum changed", { "Irssi::UI::Window", "int", NULL } },
	   { "window name changed", { "Irssi::UI::Window", NULL } },
	   { "window history changed", { "Irssi::UI::Window", "string", NULL } },
	   { "window level changed", { "Irssi::UI::Window", NULL } },
	   { "default event numeric", { "iobject", "string", "string", "string", NULL } },
	   { "message irc op_public", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message irc own_wall", { "iobject", "string", "string", NULL } },
	   { "message irc own_action", { "iobject", "string", "string", NULL } },
	   { "message irc action", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message irc own_notice", { "iobject", "string", "string", NULL } },
	   { "message irc notice", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message irc own_ctcp", { "iobject", "string", "string", "string", NULL } },
	   { "message irc ctcp", { "iobject", "string", "string", "string", "string", "string", NULL } },
	   { "message irc mode", { "iobject", "string", "string", "string", "string", NULL } },
	   { "message dcc own", { "siobject", "string", NULL } },
	   { "message dcc own_action", { "siobject", "string", NULL } },
	   { "message dcc own_ctcp", { "siobject", "string", "string", NULL } },
	   { "message dcc", { "siobject", "string", NULL } },
	   { "message dcc action", { "siobject", "string", NULL } },
	   { "message dcc ctcp", { "siobject", "string", "string", NULL } },
	   { "gui key pressed", { "int", NULL } },
	   { "paste event", { "string", "string", NULL } },
	   { "beep", { NULL } },
	   { "gui print text after finished", { "Irssi::UI::Window", "Irssi::TextUI::Line", "Irssi::TextUI::Line", "Irssi::UI::TextDest", NULL } },
	   { "gui textbuffer line removed", { "Irssi::TextUI::TextBufferView", "Irssi::TextUI::Line", "Irssi::TextUI::Line", NULL } },
	   { "gui render line text", { "Irssi::UI::TextDest", "gstring", "Irssi::UI::LineInfoMeta", NULL } },
	   { "otr event", { "iobject", "string", "string", NULL } },

    { NULL }
};
