/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.57 from the
 * contents of Dcc.xs. Do not edit this file, edit Dcc.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "../src/perl/irc/Dcc.xs"
#define PERL_NO_GET_CONTEXT
#include "module.h"

#line 14 "../src/perl/irc/Dcc.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

/* simple backcompat versions of the TARGx() macros with no optimisation */
#ifndef TARGi
#  define TARGi(iv, do_taint) sv_setiv_mg(TARG, iv)
#  define TARGu(uv, do_taint) sv_setuv_mg(TARG, uv)
#  define TARGn(nv, do_taint) sv_setnv_mg(TARG, nv)
#endif

#line 165 "../src/perl/irc/Dcc.c"

XS_EUPXS(XS_Irssi__Irc_dccs); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dccs)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
#line 10 "../src/perl/irc/Dcc.xs"
	GSList *tmp;
#line 178 "../src/perl/irc/Dcc.c"
#line 12 "../src/perl/irc/Dcc.xs"
	for (tmp = dcc_conns; tmp != NULL; tmp = tmp->next) 
		XPUSHs(sv_2mortal(simple_iobject_bless((DCC_REC *) tmp->data)));
#line 182 "../src/perl/irc/Dcc.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi__Irc_dcc_register_type); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_register_type)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "type");
    {
	char *	type = (char *)SvPV_nolen(ST(0))
;

	dcc_register_type(type);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc_dcc_unregister_type); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_unregister_type)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "type");
    {
	char *	type = (char *)SvPV_nolen(ST(0))
;

	dcc_unregister_type(type);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc_dcc_str2type); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_str2type)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "str");
    {
	char *	str = (char *)SvPV_nolen(ST(0))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = dcc_str2type(str);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Irc_dcc_type2str); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_type2str)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "type");
    {
	int	type = (int)SvIV(ST(0))
;
	char *	RETVAL;
	dXSTARG;
#line 31 "../src/perl/irc/Dcc.xs"
	RETVAL = (char *) module_find_id_str("DCC", type);
#line 254 "../src/perl/irc/Dcc.c"
	sv_setpv((SV*)TARG, RETVAL);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Irc_dcc_find_request_latest); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_find_request_latest)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "type");
    {
	int	type = (int)SvIV(ST(0))
;
	Irssi__Irc__Dcc	RETVAL;

	RETVAL = dcc_find_request_latest(type);
	{
	    SV * RETVALSV;
	    RETVALSV = simple_iobject_bless((DCC_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Irc_dcc_find_request); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_find_request)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "type, nick, arg");
    {
	int	type = (int)SvIV(ST(0))
;
	char *	nick = (char *)SvPV_nolen(ST(1))
;
	char *	arg = (char *)SvPV_nolen(ST(2))
;
	Irssi__Irc__Dcc	RETVAL;

	RETVAL = dcc_find_request(type, nick, arg);
	{
	    SV * RETVALSV;
	    RETVALSV = simple_iobject_bless((DCC_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Irc_dcc_chat_find_id); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_chat_find_id)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "id");
    {
	char *	id = (char *)SvPV_nolen(ST(0))
;
	Irssi__Irc__Dcc__Chat	RETVAL;

	RETVAL = dcc_chat_find_id(id);
	{
	    SV * RETVALSV;
	    RETVALSV = simple_iobject_bless((DCC_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Irc_dcc_chat_send); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_chat_send)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "dcc, data");
    {
	Irssi__Irc__Dcc__Chat	dcc = irssi_ref_object(ST(0))
;
	char *	data = (char *)SvPV_nolen(ST(1))
;

	dcc_chat_send(dcc, data);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc_dcc_ctcp_message); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_ctcp_message)
{
    dVAR; dXSARGS;
    if (items != 5)
       croak_xs_usage(cv,  "server, target, chat, notice, msg");
    {
	Irssi__Irc__Server	server = irssi_ref_object(ST(0))
;
	char *	target = (char *)SvPV_nolen(ST(1))
;
	Irssi__Irc__Dcc__Chat	chat = irssi_ref_object(ST(2))
;
	int	notice = (int)SvIV(ST(3))
;
	char *	msg = (char *)SvPV_nolen(ST(4))
;

	dcc_ctcp_message(server, target, chat, notice, msg);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc_dcc_get_download_path); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc_dcc_get_download_path)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "fname");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
	char *	fname = (char *)SvPV_nolen(ST(0))
;
#line 66 "../src/perl/irc/Dcc.xs"
	char *ret;
#line 390 "../src/perl/irc/Dcc.c"
#line 68 "../src/perl/irc/Dcc.xs"
	ret = dcc_get_download_path(fname);
	XPUSHs(sv_2mortal(new_pv(ret)));
	g_free(ret);
#line 395 "../src/perl/irc/Dcc.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi__Irc__Dcc_init_rec); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc__Dcc_init_rec)
{
    dVAR; dXSARGS;
    if (items != 5)
       croak_xs_usage(cv,  "dcc, server, chat, nick, arg");
    {
	Irssi__Irc__Dcc	dcc = irssi_ref_object(ST(0))
;
	Irssi__Irc__Server	server = irssi_ref_object(ST(1))
;
	Irssi__Irc__Dcc__Chat	chat = irssi_ref_object(ST(2))
;
	char *	nick = (char *)SvPV_nolen(ST(3))
;
	char *	arg = (char *)SvPV_nolen(ST(4))
;

	dcc_init_rec(dcc, server, chat, nick, arg);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc__Dcc_destroy); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc__Dcc_destroy)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "dcc");
    {
	Irssi__Irc__Dcc	dcc = irssi_ref_object(ST(0))
;

	dcc_destroy(dcc);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc__Dcc_close); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc__Dcc_close)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "dcc");
    {
	Irssi__Irc__Dcc	dcc = irssi_ref_object(ST(0))
;

	dcc_close(dcc);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Irc__Dcc_reject); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Irc__Dcc_reject)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "dcc, server");
    {
	Irssi__Irc__Dcc	dcc = irssi_ref_object(ST(0))
;
	Irssi__Irc__Server	server = irssi_ref_object(ST(1))
;

	dcc_reject(dcc, server);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Windowitem_get_dcc); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Windowitem_get_dcc)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "item");
    {
	Irssi__Windowitem	item = irssi_ref_object(ST(0))
;
	Irssi__Irc__Dcc__Chat	RETVAL;

	RETVAL = item_get_dcc(item);
	{
	    SV * RETVALSV;
	    RETVALSV = simple_iobject_bless((DCC_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}

#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_Irssi__Irc__Dcc); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_Irssi__Irc__Dcc)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        (void)newXSproto_portable("Irssi::Irc::dccs", XS_Irssi__Irc_dccs, file, "");
        (void)newXSproto_portable("Irssi::Irc::dcc_register_type", XS_Irssi__Irc_dcc_register_type, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_unregister_type", XS_Irssi__Irc_dcc_unregister_type, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_str2type", XS_Irssi__Irc_dcc_str2type, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_type2str", XS_Irssi__Irc_dcc_type2str, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_find_request_latest", XS_Irssi__Irc_dcc_find_request_latest, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_find_request", XS_Irssi__Irc_dcc_find_request, file, "$$$");
        (void)newXSproto_portable("Irssi::Irc::dcc_chat_find_id", XS_Irssi__Irc_dcc_chat_find_id, file, "$");
        (void)newXSproto_portable("Irssi::Irc::dcc_chat_send", XS_Irssi__Irc_dcc_chat_send, file, "$$");
        (void)newXSproto_portable("Irssi::Irc::dcc_ctcp_message", XS_Irssi__Irc_dcc_ctcp_message, file, "$$$$$");
        (void)newXSproto_portable("Irssi::Irc::dcc_get_download_path", XS_Irssi__Irc_dcc_get_download_path, file, "$");
        (void)newXSproto_portable("Irssi::Irc::Dcc::init_rec", XS_Irssi__Irc__Dcc_init_rec, file, "$$$$$");
        (void)newXSproto_portable("Irssi::Irc::Dcc::destroy", XS_Irssi__Irc__Dcc_destroy, file, "$");
        (void)newXSproto_portable("Irssi::Irc::Dcc::close", XS_Irssi__Irc__Dcc_close, file, "$");
        (void)newXSproto_portable("Irssi::Irc::Dcc::reject", XS_Irssi__Irc__Dcc_reject, file, "$$");
        (void)newXSproto_portable("Irssi::Windowitem::get_dcc", XS_Irssi__Windowitem_get_dcc, file, "$");
#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
