/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.57 from the
 * contents of Server.xs. Do not edit this file, edit Server.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "../src/perl/common/Server.xs"
#define PERL_NO_GET_CONTEXT
#include "module.h"

#line 14 "../src/perl/common/Server.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

/* simple backcompat versions of the TARGx() macros with no optimisation */
#ifndef TARGi
#  define TARGi(iv, do_taint) sv_setiv_mg(TARG, iv)
#  define TARGu(uv, do_taint) sv_setuv_mg(TARG, uv)
#  define TARGn(nv, do_taint) sv_setnv_mg(TARG, nv)
#endif

#line 165 "../src/perl/common/Server.c"

XS_EUPXS(XS_Irssi_servers); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_servers)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
#line 10 "../src/perl/common/Server.xs"
	GSList *tmp;
#line 178 "../src/perl/common/Server.c"
#line 12 "../src/perl/common/Server.xs"
	for (tmp = servers; tmp != NULL; tmp = tmp->next) {
		XPUSHs(sv_2mortal(iobject_bless((SERVER_REC *) tmp->data)));
	}
#line 183 "../src/perl/common/Server.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi_reconnects); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_reconnects)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
#line 19 "../src/perl/common/Server.xs"
	GSList *tmp;
#line 201 "../src/perl/common/Server.c"
#line 21 "../src/perl/common/Server.xs"
	for (tmp = reconnects; tmp != NULL; tmp = tmp->next) {
		XPUSHs(sv_2mortal(plain_bless(tmp->data, "Irssi::Reconnect")));
	}
#line 206 "../src/perl/common/Server.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi_chatnets); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_chatnets)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    PERL_UNUSED_VAR(ax); /* -Wall */
    SP -= items;
    {
#line 28 "../src/perl/common/Server.xs"
	GSList *tmp;
#line 224 "../src/perl/common/Server.c"
#line 30 "../src/perl/common/Server.xs"
	for (tmp = chatnets; tmp != NULL; tmp = tmp->next) {
		XPUSHs(sv_2mortal(iobject_bless((CHATNET_REC *) tmp->data)));
	}
#line 229 "../src/perl/common/Server.c"
	PUTBACK;
	return;
    }
}


XS_EUPXS(XS_Irssi_server_create_conn); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_server_create_conn)
{
    dVAR; dXSARGS;
    if (items < 3 || items > 6)
       croak_xs_usage(cv,  "chat_type, dest, port, chatnet=NULL, password=NULL, nick=NULL");
    {
	int	chat_type = (int)SvIV(ST(0))
;
	char *	dest = (char *)SvPV_nolen(ST(1))
;
	int	port = (int)SvIV(ST(2))
;
	char *	chatnet;
	char *	password;
	char *	nick;
	Irssi__Connect	RETVAL;

	if (items < 4)
	    chatnet = NULL;
	else {
	    chatnet = (char *)SvPV_nolen(ST(3))
;
	}

	if (items < 5)
	    password = NULL;
	else {
	    password = (char *)SvPV_nolen(ST(4))
;
	}

	if (items < 6)
	    nick = NULL;
	else {
	    nick = (char *)SvPV_nolen(ST(5))
;
	}

	RETVAL = server_create_conn(chat_type, dest, port, chatnet, password, nick);
	{
	    SV * RETVALSV;
	    RETVALSV = iobject_bless((SERVER_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_server_find_tag); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_server_find_tag)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "tag");
    {
	char *	tag = (char *)SvPV_nolen(ST(0))
;
	Irssi__Server	RETVAL;

	RETVAL = server_find_tag(tag);
	{
	    SV * RETVALSV;
	    RETVALSV = iobject_bless((SERVER_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_server_find_chatnet); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_server_find_chatnet)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "chatnet");
    {
	char *	chatnet = (char *)SvPV_nolen(ST(0))
;
	Irssi__Server	RETVAL;

	RETVAL = server_find_chatnet(chatnet);
	{
	    SV * RETVALSV;
	    RETVALSV = iobject_bless((SERVER_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi_chatnet_find); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi_chatnet_find)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "name");
    {
	char *	name = (char *)SvPV_nolen(ST(0))
;
	Irssi__Chatnet	RETVAL;

	RETVAL = chatnet_find(name);
	{
	    SV * RETVALSV;
	    RETVALSV = iobject_bless((SERVER_REC *)RETVAL);
	    RETVALSV = sv_2mortal(RETVALSV);
	    ST(0) = RETVALSV;
	}
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Server_disconnect); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_disconnect)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "server");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;

	server_disconnect(server);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Server_ref); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_ref)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "server");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;

	server_ref(server);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Server_unref); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_unref)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "server");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;

	server_unref(server);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Server_isnickflag); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_isnickflag)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "server, flag");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char	flag = (char)*SvPV_nolen(ST(1))
;
	int	RETVAL;
	dXSTARG;
#line 76 "../src/perl/common/Server.xs"
	RETVAL = server->isnickflag(server, flag);
#line 419 "../src/perl/common/Server.c"
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Server_ischannel); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_ischannel)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "server, data");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char *	data = (char *)SvPV_nolen(ST(1))
;
	int	RETVAL;
	dXSTARG;
#line 85 "../src/perl/common/Server.xs"
	RETVAL = server->ischannel(server, data);
#line 442 "../src/perl/common/Server.c"
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Server_get_nick_flags); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_get_nick_flags)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "server");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char *	RETVAL;
	dXSTARG;
#line 93 "../src/perl/common/Server.xs"
	RETVAL = (char *) server->get_nick_flags(server);
#line 463 "../src/perl/common/Server.c"
	sv_setpv((SV*)TARG, RETVAL);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_Irssi__Server_send_message); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_send_message)
{
    dVAR; dXSARGS;
    if (items != 4)
       croak_xs_usage(cv,  "server, target, msg, target_type");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char *	target = (char *)SvPV_nolen(ST(1))
;
	char *	msg = (char *)SvPV_nolen(ST(2))
;
	int	target_type = (int)SvIV(ST(3))
;
#line 104 "../src/perl/common/Server.xs"
	server->send_message(server, target, msg, target_type);
#line 488 "../src/perl/common/Server.c"
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Server_meta_stash); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_meta_stash)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "server, meta_key, meta_value");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char *	meta_key = (char *)SvPV_nolen(ST(1))
;
	char *	meta_value = (char *)SvPV_nolen(ST(2))
;

	server_meta_stash(server, meta_key, meta_value);
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_Irssi__Server_meta_stash_find); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_Irssi__Server_meta_stash_find)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "server, meta_key");
    {
	Irssi__Server	server = irssi_ref_object(ST(0))
;
	char *	meta_key = (char *)SvPV_nolen(ST(1))
;
	char *	RETVAL;
	dXSTARG;
#line 117 "../src/perl/common/Server.xs"
	RETVAL = (char *) server_meta_stash_find(server, meta_key);
#line 529 "../src/perl/common/Server.c"
	sv_setpv((SV*)TARG, RETVAL);
	ST(0) = TARG;
    }
    XSRETURN(1);
}

#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_Irssi__Server); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_Irssi__Server)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        (void)newXSproto_portable("Irssi::servers", XS_Irssi_servers, file, "");
        (void)newXSproto_portable("Irssi::reconnects", XS_Irssi_reconnects, file, "");
        (void)newXSproto_portable("Irssi::chatnets", XS_Irssi_chatnets, file, "");
        (void)newXSproto_portable("Irssi::server_create_conn", XS_Irssi_server_create_conn, file, "$$$;$$$");
        (void)newXSproto_portable("Irssi::server_find_tag", XS_Irssi_server_find_tag, file, "$");
        (void)newXSproto_portable("Irssi::server_find_chatnet", XS_Irssi_server_find_chatnet, file, "$");
        (void)newXSproto_portable("Irssi::chatnet_find", XS_Irssi_chatnet_find, file, "$");
        (void)newXSproto_portable("Irssi::Server::disconnect", XS_Irssi__Server_disconnect, file, "$");
        (void)newXSproto_portable("Irssi::Server::ref", XS_Irssi__Server_ref, file, "$");
        (void)newXSproto_portable("Irssi::Server::unref", XS_Irssi__Server_unref, file, "$");
        (void)newXSproto_portable("Irssi::Server::isnickflag", XS_Irssi__Server_isnickflag, file, "$$");
        (void)newXSproto_portable("Irssi::Server::ischannel", XS_Irssi__Server_ischannel, file, "$$");
        (void)newXSproto_portable("Irssi::Server::get_nick_flags", XS_Irssi__Server_get_nick_flags, file, "$");
        (void)newXSproto_portable("Irssi::Server::send_message", XS_Irssi__Server_send_message, file, "$$$$");
        (void)newXSproto_portable("Irssi::Server::meta_stash", XS_Irssi__Server_meta_stash, file, "$$$");
        (void)newXSproto_portable("Irssi::Server::meta_stash_find", XS_Irssi__Server_meta_stash_find, file, "$$");
#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
